<?php

namespace Drupal\mm_media\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\editor\Ajax\EditorDialogSave;
use Drupal\editor\EditorInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\file\Entity\File;
use Drupal\media\Entity\Media;
use Drupal\media_library\MediaLibraryState;
use Drupal\mm_media\Plugin\EntityBrowser\Widget\MMTree;
use Drupal\monster_menus\Constants;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

class MMMediaLibrary extends FormBase {

  /**
   * The entity repository.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * The entity display repository.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected $entityDisplayRepository;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a EditorMediaDialog object.
   *
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   */
  public function __construct(EntityRepositoryInterface $entity_repository, EntityDisplayRepositoryInterface $entity_display_repository, EntityTypeManagerInterface $entity_type_manager) {
    $this->entityRepository = $entity_repository;
    $this->entityDisplayRepository = $entity_display_repository;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.repository'),
      $container->get('entity_display.repository'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * @inheritDoc
   */
  public function getFormId() {
    return 'mm_media_library';
  }

  /**
   * @param array $form
   *   A nested array form elements comprising the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param \Drupal\editor\EditorInterface|null $editor
   *   The text editor to which this dialog corresponds.
   */
  public function buildForm(array $form, FormStateInterface $form_state, EditorInterface $editor = NULL) {
    $this_mmtid = $this->getRouteMatch()->getParameter('mm_tree') ?: mm_home_mmtid();
    $pop_start = implode('/', mm_content_get_parents_with_self($this_mmtid));

    $form['#attached']['library'][] = 'mm_media/mmmedialibrary_editing';
    $form['#attached']['library'][] = 'editor/drupal.editor.dialog';

    $state = MediaLibraryState::fromRequest($this->getRequest());
    $allowed_types = implode(',', $state->getAllowedTypeIds()) . ',' . MMTree::FILE2MEDIA_TYPE;

    $form['browser'] = [
      '#title' => $this->t('Media'),
      '#type' => 'mm_medialist',
      '#default_value' => ['' => $this->t('(choose a media item)')],
      '#mm_list_popup_start' => $pop_start,
      '#mm_list_selectable' => Constants::MM_PERMS_READ,
      '#mm_list_file_types' => $allowed_types,
      '#mm_list_min' => 1,
      '#mm_list_max' => 1,
      '#mm_list_auto_choose' => TRUE,
      '#mm_list_instance_start' => 100,
      '#mm_list_hide_left_pane' => TRUE,
      '#ajax' => [
        'callback' => '::updatePreview',
        'event' => 'change',
        'wrapper' => 'mmmedialibrary-preview',
      ]
    ];
    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['insert'] = array(
      '#type' => 'button',
      '#value' => t('Insert'),
      '#button_type' => 'button',
      '#ajax' => [
        'callback' => '::submitForm',
        'event' => 'click',
      ],
    );
    $form['actions']['cancel'] = array(
      '#type' => 'button',
      '#value' => t('Cancel'),
      '#button_type' => 'button',
    );

    $current_view_mode = $form_state->getValue(['attributes', 'data-view-mode']) ?:
      $editor->getFilterFormat()->filters()->get('media_embed')->settings['default_view_mode'];
    $attributes = [
      '#tree' => TRUE,
      'data-view-mode' => [
        '#type' => 'hidden',
        '#value' => $current_view_mode,
      ],
    ];
    $preview = [];
    if ($media = $this->getMedia($form_state)) {
      $view_mode_options = array_intersect_key($this->entityDisplayRepository->getViewModeOptionsByBundle($media->getEntityTypeId(), $media->bundle()), $editor->getFilterFormat()->filters()->get('media_embed')->settings['allowed_view_modes']);
      $attributes['data-view-mode'] = [
        '#title' => $this->t('Display'),
        '#type' => 'select',
        '#options' => $view_mode_options,
        '#default_value' => $current_view_mode,
        '#access' => count($view_mode_options) >= 2,
        '#ajax' => [
          'callback' => '::updatePreview',
          'event' => 'change',
          'wrapper' => 'mmmedialibrary-preview',
        ]
      ];
      $preview = $this->entityTypeManager
        ->getViewBuilder($media->getEntityTypeId())
        ->view($media, $current_view_mode);
    }
    $form['preview'] = [
      '#prefix' => '<div id="mmmedialibrary-preview" style="clear: both">',
      'attributes' => $attributes,
      'preview' => $preview,
      '#suffix' => '</div>',
    ];

    return $form;
  }

  /**
   * @inheritDoc
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $response = new AjaxResponse();

    // When the `alt` attribute is set to two double quotes, transform it to the
    // empty string: two double quotes signify "empty alt attribute". See above.
    if (trim($form_state->getValue(['attributes', 'alt'], '')) === '""') {
      $form_state->setValue(['attributes', 'alt'], '""');
    }

    // The `alt` attribute is optional: if it isn't set, the default value
    // simply will not be overridden. It's important to set it to FALSE
    // instead of unsetting the value.  This way we explicitly inform
    // the client side about the new value.
    if ($form_state->hasValue(['attributes', 'alt']) && trim($form_state->getValue(['attributes', 'alt'])) === '') {
      $form_state->setValue(['attributes', 'alt'], FALSE);
    }

    // If the selected view mode matches the default on the filter, remove the
    // attribute.
    if (!empty($form_state->get('filter_default_view_mode')) && $form_state->getValue(['attributes', 'data-view-mode']) === $form_state->get('filter_default_view_mode')) {
      $form_state->setValue(['attributes', 'data-view-mode'], FALSE);
    }

    if ($media = $this->getMedia($form_state)) {
      if ($media->isNew()) {
        // ::getMedia() created a temporary Media item, so save it now.
        $media->save();
      }
      // Only send back the relevant values.
      $values = [
        'hasCaption' => $form_state->getValue('hasCaption'),
        'attributes' => $form_state->getValue('attributes') + [
          'data-entity-type' => $media->getEntityTypeId(),
          'data-entity-uuid' => $media->uuid(),
          'data-align' => 'left',
        ],
      ];
      $response->addCommand(new EditorDialogSave($values));
    }
    $response->addCommand(new CloseModalDialogCommand());

    return $response;
  }

  public function updatePreview(array &$form, FormStateInterface $form_state, Request $request) {
    return $form['preview'];
  }

  private function getMedia(FormStateInterface $form_state) {
    if ($item = mm_ui_mmlist_key0($form_state->getValue('browser'))) {
      [, $fid, $mid] = explode('/', $item);
      if ($file = File::load($fid)) {
        if ($mid > 0) {
          return Media::load($mid);
        }
        // Create a temporary media item.
        $type = explode('/', $file->getMimeType())[0];
        $media = Media::create(['bundle' => $type]);
        foreach ($media->getFieldDefinitions() as $field) {
          if ($field instanceof FieldConfig && $field->getFieldStorageDefinition()->getSetting('target_type') === 'file') {
            $media->set($field->getName(), ['target_id' => $file->id()]);
            break;
          }
        }
        return $media;
      }
    }
  }

}
